package im.getsocial;

import android.util.Log;

import androidx.annotation.NonNull;

import im.getsocial.sdk.Callback;
import im.getsocial.sdk.json.GetSocialJson;
import im.getsocial.sdk.GetSocial;
import im.getsocial.sdk.communities.CurrentUser;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.EventChannel.EventSink;
import io.flutter.plugin.common.EventChannel.StreamHandler;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** GetSocialPlugin */
public class GetSocialPlugin implements FlutterPlugin, MethodCallHandler, StreamHandler {
  private static final String TAG = "GetSocialPlugin";

  private MethodChannel methodsChannel;
  private EventChannel onInitializeChannel;
  private EventChannel onCurrentUserChangedChannel;
  private EventChannel onReferralDataReceivedChannel;
  private EventChannel onNotificationReceivedChannel;
  private EventChannel onNotificationClickedChannel;
  private EventChannel onNotificationTokenReceivedChannel;

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    Log.d(TAG, "attached to engine");
    registerChannels(flutterPluginBinding.getBinaryMessenger());
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    Log.d(TAG, "invoked registerWith");
    new GetSocialPlugin().registerChannels(registrar.messenger());
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, final @NonNull Result result) {
    String name = call.method;
    String method = call.hasArgument("method") ? call.<String>argument("method") : null;
    String body = call.hasArgument("body") ? call.<String>argument("body") : null;
    if (body != null) {
      body = body.replaceAll("GETSOCIAL_CURRENT_USER_PLACEHOLDER_42_42", GetSocial.getCurrentUser().getId());
    }
    switch (name) {
      case "callSync":
        result.success(GetSocialJson.callSync(method, body));
        break;
      case "callAsync":
        GetSocialJson.callAsync(method, body, new Callback<String>() {
          @Override
          public void onSuccess(String s) {
            result.success(s);
          }
        }, new Callback<String>() {
          @Override
          public void onSuccess(String s) {
            result.error("0", s, null);
          }
        });
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    Log.d(TAG,"detached from engine");
    methodsChannel.setMethodCallHandler(null);
    methodsChannel = null;
    // sending null handler to deregister
    onInitializeChannel.setStreamHandler(null);
    onInitializeChannel = null;
    onCurrentUserChangedChannel.setStreamHandler(null);
    onCurrentUserChangedChannel = null;
    onReferralDataReceivedChannel.setStreamHandler(null);
    onReferralDataReceivedChannel = null;
    onNotificationReceivedChannel.setStreamHandler(null);
    onNotificationReceivedChannel = null;
    onNotificationClickedChannel.setStreamHandler(null);
    onNotificationClickedChannel = null;
    onNotificationTokenReceivedChannel.setStreamHandler(null);
    onNotificationTokenReceivedChannel = null;

  }

  @Override
  public void onListen(Object arguments, final EventSink events) {
    if (arguments instanceof String) {
      final String listener = (String) arguments;
      Log.d(TAG, "start listening for " + listener);
      GetSocialJson.registerListener(listener, new Callback<String>() {
        @Override
        public void onSuccess(String s) {
          events.success(s);
        }
      });
    }
  }

  @Override
  public void onCancel(Object arguments) {
  }

  private void registerChannels(BinaryMessenger messenger) {
    Log.d(TAG, "registering channels");
    methodsChannel = new MethodChannel(messenger, "im.getsocial.flutter/methods");
    methodsChannel.setMethodCallHandler(this);
    onInitializeChannel = new EventChannel(messenger, "im.getsocial.flutter/event/initialization");
    onInitializeChannel.setStreamHandler(this);
    onCurrentUserChangedChannel = new EventChannel(messenger, "im.getsocial.flutter/event/currentuserchanged");
    onCurrentUserChangedChannel.setStreamHandler(this);
    onReferralDataReceivedChannel = new EventChannel(messenger, "im.getsocial.flutter/event/referraldatareceived");
    onReferralDataReceivedChannel.setStreamHandler(this);
    onNotificationReceivedChannel = new EventChannel(messenger, "im.getsocial.flutter/event/notificationreceived");
    onNotificationReceivedChannel.setStreamHandler(this);
    onNotificationClickedChannel = new EventChannel(messenger, "im.getsocial.flutter/event/notificationclicked");
    onNotificationClickedChannel.setStreamHandler(this);
    onNotificationTokenReceivedChannel = new EventChannel(messenger, "im.getsocial.flutter/event/notificationtokenreceived");
    onNotificationTokenReceivedChannel.setStreamHandler(this);
  }
}
