import '../common/action.dart';
import '../common/media_attachment.dart';
import '../src/utils.dart';
import '../usermanagement/user.dart';
import 'notificationbutton.dart';
import 'notificationcustomization.dart';

class NotificationStatus {
  /// Notification is read.
  static const String read = 'read';

  /// Notification is unread.
  static const String unread = 'unread';

  /// Notification is consumed.
  static const String consumed = 'consumed';

  /// Notification is ignored.
  static const String ignored = 'ignored';
}

class NotificationType {
  /// Someone commented on your activity.
  static const String comment = 'comment';

  /// Someone liked your activity.
  static const String likeActivity = 'activity_like';

  /// Someone liked your comment.
  static const String likeComment = 'comment_like';

  /// Someone commented on the activity where you've commented before.
  static const String relatedComment = 'related_comment';

  /// You became friends with another user.
  static const String newFriendship = 'friends_add';

  /// Someone accepted your invite.
  static const String inviteAccepted = 'invite_accept';

  /// Someone mentioned you in comment.
  static const String mentionInComment = 'comment_mention';

  /// Someone mentioned you in activity.
  static const String mentionInActivity = 'activity_mention';

  /// Someone replied to your comment.
  static const String replyToComment = 'comment_reply';

  /// Smart targeting Push Notifications.
  static const String targeting = 'targeting';

  /// Notifications sent from the Dashboard when using "Test Push Notifications".
  static const String direct = 'direct';

  /// Notification sent from SDK.
  static const String sdk = 'custom';
}

/// Notification object.
class GetSocialNotification {
  GetSocialNotification.fromJSON(Map<String, dynamic> json)
      : id = json['id'],
        type = json['type'],
        status = json['status'],
        createdAt = json['createdAt'],
        text = json['text'],
        title = json['title'],
        mediaAttachment = json['mediaAttachment'] == null
            ? null
            : MediaAttachment.fromJSON(json['mediaAttachment']),
        notificationAction = json['action'] == null
            ? null
            : GetSocialAction.fromJSON(json['action']),
        actionButtons = json['actionButtons'] == null
            ? []
            : Utils.createList(json['actionButtons'],
                (jsonMap) => NotificationButton.fromJSON(jsonMap)),
        sender = json['sender'] == null ? null : User.fromJSON(json['sender']),
        customization = json['customization'] == null
            ? null
            : NotificationCustomization.fromJSON(json['customization']);

  /// Id
  String id;

  /// Type, check NotificationType for possible values.
  String type;

  /// Status, check NotificationStatus for possible values.
  String status;

  /// Creation date.
  int createdAt;

  /// Text.
  String? text;

  /// Title.
  String? title;

  /// Attached media.
  MediaAttachment? mediaAttachment;

  /// Action.
  GetSocialAction? notificationAction;

  /// Action buttons.
  List<NotificationButton> actionButtons;

  /// Sender.
  User? sender;

  /// Customization.
  NotificationCustomization? customization;

  @override
  String toString() {
    var senderId = this.sender == null ? 'null' : this.sender?.userId;
    return 'GetSocialNotification{'
        'id: $id, '
        'type: $type,'
        'status: $status,'
        'createdAt: $createdAt,'
        'text: $text,'
        'title: $title,'
        'mediaAttachment: $mediaAttachment,'
        'notificationAction: $notificationAction,'
        'actionButtons: $actionButtons,'
        'sender: $senderId,'
        'customization: $customization,'
        '}';
  }
}
