import '../../common/codable.dart';

/// Query to get notifications.
class NotificationsQuery extends Encodable {
  /// Types, check NotificationType for possible values.
  List<String> types = [];

  /// Statuses, check NotificationStatus for possible values.
  List<String> statuses = [];

  /// Actions, check NotificationType for possible values.
  List<String> actions = [];

  /// Create query to get notifications with any statuses.
  static NotificationsQuery withAllStatuses() {
    var query = NotificationsQuery();
    query.statuses = [];
    return query;
  }

  /// Create query to get notifications with specified statuses.
  ///
  /// [statuses]  List of statuses.
  static NotificationsQuery withStatuses(List<String> statuses) {
    var query = NotificationsQuery();
    query.statuses = statuses;
    return query;
  }

  /// Append type to query.
  ///
  /// [type]    Notification type.
  NotificationsQuery appendType(String type) {
    this.types.add(type);
    return this;
  }

  /// Append status to query.
  ///
  /// [status]    Notification status.
  NotificationsQuery appendStatus(String status) {
    this.statuses.add(status);
    return this;
  }

  /// Append action to query.
  ///
  /// [action]    Notification action.
  NotificationsQuery appendAction(String action) {
    this.actions.add(action);
    return this;
  }

  @override
  String toString() {
    return 'NotificationsQuery{'
        'types: $types, '
        'statuses: $statuses, '
        'actions: $actions, '
        '}';
  }

  Map<String, dynamic> toJSON() =>
      {'types': types, 'statuses': statuses, 'actions': actions};
}
